from TuneCoach.gui.constants import *
import tkinter as tk


class MoreInfoWindow:
    @staticmethod
    def refresh(window, mainWindow):
        window.destroy()
        MoreInfoWindow(mainWindow)

    def __init__(self, mainWindow):
        my_window = tk.Toplevel(mainWindow.master)
        currentSession = mainWindow.session
        data = currentSession.data

        topFrame = tk.Frame(my_window, bg=background_color)
        middleFrame = tk.Frame(my_window, bg=background_color)
        bottomFrame = tk.Frame(my_window, bg=background_color)

        topFrame.grid(row=0, sticky="nsew")
        middleFrame.grid(row=1, sticky="nsew")
        bottomFrame.grid(row=2, sticky="nsew")

        final_string = ""
        title_string = ""
        if currentSession is None:
            title_string += "No Session Currently Active"
        else:
            title_string += currentSession.name + " Information:"
            if data._overall_count > 0:
                avg_cents = data._cents / data._overall_count
                for i in range(12):
                    if data._pitch_count[i] == 0:
                        final_string += (data._notes[i] + " was not played/sung in the session.\n")
                    else:
                        pitch_error = (100.0 * data._in_tune_count[i]) / data._pitch_count[i]
                        final_string += ("%s was in tune for %.2f %% of the time.\n" % (
                        data._notes[i], pitch_error))
                final_string += "\nOverall"
                final_string += "\nYou were in tune for %.2f %% of the time." % data.get_overall()
                final_string += "\nYou were off by an average of %.2f cents." % avg_cents
            else:
                final_string += "no input yet"

        titleLabel = tk.Label(topFrame, text=title_string, bg=background_color, fg="white")
        titleLabel.pack()
        my_label = tk.Label(middleFrame, text=final_string, bg="#B0AFAF", fg="black", bd=5, relief=tk.RIDGE)
        my_label.pack()
        exit_button = tk.Button(bottomFrame, text="Exit", command=lambda: my_window.destroy())
        exit_button.pack(side=tk.LEFT, padx=5, pady=5)
        refresh_button = tk.Button(bottomFrame, text="Refresh", command=lambda: self.refresh(my_window, mainWindow))
        refresh_button.pack(side=tk.RIGHT, padx=5, pady=5)
        my_window.lift(mainWindow.master)



import tkinter as tk
import PIL.Image
import PIL.ImageTk
import TuneCoach.gui as gui
from importlib.resources import open_binary

# loads piano.jpeg using importlib's resources
# feature
image = open_binary(gui, 'piano.jpeg')


# Different classes for pop-up windows.
class SessionHistory:
    def create_circle(self, x, y, r, canvasName, fillColor):  # center coordinates, radius
        x0 = x - r
        y0 = y - r
        x1 = x + r
        y1 = y + r
        return canvasName.create_oval(x0, y0, x1, y1, fill=fillColor)

    def __init__(self, mainWindow, workingFrame):
        self.mainWindow = mainWindow
        self.frame = workingFrame
        self.canvas = tk.Canvas(workingFrame, bg="#bdd0df")
        self.canvas.pack(expand=True, fill=tk.BOTH)

        self.width = self.frame.winfo_width()
        self.height = self.frame.winfo_height()

        self.large_image = PIL.Image.open(image)
        self.aspect_ratio = self.large_image.width / self.large_image.height
        self.piano_image = None

        self.available_width = self.width
        self.circle_size = self.available_width/65
        self.circle_start = 0
        self.circle_list = [None] * 64  # TODO: don't hardcode size and coordinate with Feedback buffer

        self.frame.bind("<Configure>", self.setup)

    def setup(self, event):
        self.clear()
        self.canvas.delete("all")

        self.width = self.frame.winfo_width()
        self.height = self.frame.winfo_height()

        resized = self.large_image.resize((int(self.height*self.aspect_ratio), int(self.height)), PIL.Image.ANTIALIAS)
        self.piano_image = PIL.ImageTk.PhotoImage(resized)
        self.canvas.create_image(0, 0, anchor=tk.NW, image=self.piano_image)

        self.available_width = self.width - self.piano_image.width()
        self.circle_start = self.piano_image.width()
        self.circle_size = 0.5*self.available_width/64
        
        self.noteDict = {
            "C": self.height / 14,
            "C#": self.height / 7,
            "D": self.height / 14 * 3,
            "D#": self.height / 7 * 2,
            "E": self.height / 14 * 5,
            "F": self.height / 14 * 7,
            "F#": self.height / 7 * 4,
            "G": self.height / 14 * 9,
            "G#": self.height / 7 * 5,
            "A": self.height / 14 * 11,
            "A#": self.height / 7 * 6,
            "B": self.height / 14 * 13
        }

        for note in self.noteDict:
            self.canvas.create_line(self.piano_image.width(), self.noteDict[note], self.width, self.noteDict[note], width=3)

    def update(self, data, force=False):
        if data is not None and (force or data.has_new_data):
            data.has_new_data = False
            recent = list(data.display_buffer)
            for i, (note, cents) in enumerate(recent):
                color = "red"
                if abs(cents) <= self.mainWindow.threshold:
                    color = "green"
                elif abs(cents) <= self.mainWindow.yellow_threshold:
                    color = "yellow"

                circle = self.circle_list[i]
                x = self.circle_start + self.circle_size/2 + 2*self.circle_size*i
                y = self.noteDict[note]
                if circle is None:
                    c = self.create_circle(x, y, self.circle_size, self.canvas, color)
                    self.circle_list[i] = c
                else:
                    self.canvas.coords(circle, x - self.circle_size, y - self.circle_size, x + self.circle_size, y + self.circle_size)
                    self.canvas.itemconfig(circle, fill=color)

    def clear(self):
        for circle in self.circle_list:
            self.canvas.delete(circle)
        self.circle_list = [None] * 64



import tkinter as tk
from TuneCoach.python_bridge.SessionData import *
from TuneCoach.gui.constants import *
from TuneCoach.python_bridge.AudioManager import *
from TuneCoach.gui.SaveProgressWindow import *


# Settings window to create a new session
class NewSessionWindow:
    #will check whether or not new session name is valid and whether or not you want to save your old session
    def new_session_buffer(self, mainWindow, popup, newName):
        x = 0
        for session in mainWindow.currentPracticeSession.get_existing_sessions():
            if mainWindow.currentPracticeSession.load_from_file(session)._name == newName or mainWindow.currentPracticeSession.load_from_file(session)._name == newName + ".session" or newName == "Temporary Practice Session" or newName == mainWindow.currentPracticeSession._name:
                self.text_entry_label.configure(text = "Session already exists. \n Please enter a new session name.")
                x += 1
        if x == 0:
            if mainWindow.currentPracticeSession is not None and mainWindow.currentPracticeSession._name != "Temporary Session":
                SaveProgressWindow(mainWindow, self, popup, newName)
            else:
                self.creating_a_new_session(mainWindow, popup, newName)
    #will actually create new session
    def creating_a_new_session(self, mainWindow, popup, newName):
        # Cleanup Code:
        #x = 0
        #for session in mainWindow.currentPracticeSession.get_existing_sessions():
        #    if mainWindow.currentPracticeSession.load_from_file(session)._name == newName or mainWindow.currentPracticeSession.load_from_file(session)._name == newName + ".session":
        #        self.text_entry_label.configure(text = "Session already exists.\n Please enter a new session name.")
        #        print("Session Already Exists")
        #        x += 1
        #if x == 0:
        #    if mainWindow.currentPracticeSession is not None and mainWindow.currentPracticeSession._name != "Temporary Session":
        #        SaveProgressWindow(mainWindow)
            if mainWindow.audio_manager is not None:
                mainWindow.audio_manager.destroy()

            # New Session Code:
            mainWindow.reset_everything()
            new_session = Session(mainWindow.threshold, newName) # TODO: Make threshold setting in the popup for new session
            mainWindow.currentPracticeSession = new_session
            mainWindow.practiceSessionList.append(new_session)
            mainWindow.audio_manager = AudioManager(new_session)
            print("new session is created, it is set to our current practice session variable")
            mainWindow.myDiagnosticObject.sessionName.configure(text=newName)
            mainWindow.myDiagnosticObject.update_plot(-1)
            mainWindow.myHistoryObject.clear()
            popup.destroy()

    def __init__(self, mainWindow):
        new_sesh_window = tk.Toplevel(mainWindow.master)
        new_sesh_window.geometry("500x100")
        top_frame = tk.Frame(new_sesh_window, bd=5, bg=background_color)
        left_frame = tk.Frame(new_sesh_window, bd=5, bg=background_color)
        middle_frame = tk.Frame(new_sesh_window, bd=5, bg=background_color)
        right_frame = tk.Frame(new_sesh_window, bd=5, bg=background_color)

        top_frame.grid(column=0, row=0, columnspan=3, sticky="nsew")
        left_frame.grid(row=1, column=0, sticky="nsew")
        middle_frame.grid(row=1, column=1, sticky="nsew")
        right_frame.grid(row=1, column=2, sticky="nsew")

        new_sesh_window.grid_rowconfigure(0, weight=1)
        new_sesh_window.grid_rowconfigure(1, weight=3)
        new_sesh_window.grid_columnconfigure(0, weight=1)
        new_sesh_window.grid_columnconfigure(1, weight=1)
        new_sesh_window.grid_columnconfigure(2, weight=1)

        # will sub out these stand-ins for values once we get set up how and where we will store practice sessions.

        create_session_label = tk.Label(top_frame, text="Create New Session", bg=background_color, fg="white")
        create_session_label.pack()
        new_sesh_window.lift(mainWindow.master)
        self.text_entry_label = tk.Label(left_frame, text="Enter name of new Session", fg="white", bg=background_color)
        self.text_entry_label.pack()
        text_entry = tk.Entry(middle_frame)
        if  mainWindow.currentPracticeSession._name != "new-session-%d" % len(mainWindow.currentPracticeSession.get_existing_sessions()):
            defaultText = "new-session-%d" % len(mainWindow.currentPracticeSession.get_existing_sessions())
        else:
            defaultText = "new-session-%d" % (len(mainWindow.currentPracticeSession.get_existing_sessions())+1)
        text_entry.insert(tk.END, defaultText)
        text_entry.pack()
        #enter_entry = tk.Button(right_frame, text="Enter",
                               #command=lambda: self.creating_a_new_session(mainWindow, new_sesh_window, text_entry.get()))
        enter_entry = tk.Button(right_frame, text = "Enter", command = lambda : self.new_session_buffer(mainWindow, new_sesh_window, text_entry.get()))
        enter_entry.pack()

        new_sesh_window.lift(mainWindow.master)



from TuneCoach.gui.MainWindow import *
from TuneCoach.gui.PitchDisplay import *
from tkinter import *
from TuneCoach.python_bridge.AudioManager import *
import time


def space_pressed(event, mainWindow):
    mainWindow.toggle_pause()

    # end = time.time()
    # elapsed_time = end - start
    # minutes = int(elapsed_time) // 60
    # seconds = int(elapsed_time) % 60

    # print("Here are the results of this session:")
    # print("-------------------------------------")
    # if minutes == 0:
    #     print("This session lasted", seconds, "seconds.")
    # elif minutes == 1:
    #     print("This session lasted", minutes, "minute and", seconds, "seconds.")
    # else:
    #     print("This session lasted", minutes, "minutes and", seconds, "seconds.")
    # print("")


def cleanup(mainWindow):
    if not mainWindow.save_practice_session(ask=True):
        return # do not close if we're saving and then we cancel
    if mainWindow.audio_manager is not None:
        mainWindow.audio_manager.destroy()
    mainWindow.master.destroy()



def main():
    start = time.time()
    root = Tk()
    root.title("TuneCoach")
    our_window = MainWindow(root)
    root.bind('<space>', lambda event, arg=our_window: space_pressed(event, arg))
    root.wm_protocol("WM_DELETE_WINDOW", lambda w=our_window: cleanup(w))
    root.mainloop()


if __name__ == "__main__":
    main()



import tkinter as tk
from TuneCoach.gui.constants import *


class TutorialWindow:
    def __init__(self, mainWindow):
        tutorial_window = tk.Toplevel(mainWindow.master)
        tutorial_window.geometry()

        message = "                                             Welcome to TuneCoach!\n" \
                  "TuneCoach is convenient PC-based tuning application that records sessions for intonation\n" \
                  "and provides detailed feedback for your performance. \n" \
                  "\n" \
                  "To get started, click the \"File\" dropdown menu in the top left corner and select \n" \
                  "\"New Practice Session\"\n" \
                  "In the same dropdown menu, there are more options to manage your tuning sessions\n" \
                  "(Note: The pitch detector will function upon startup, but it will be in practice mode.\n" \
                  "This means that the tuner works but no session is started and therefore no data is saved.)\n" \
                  "\n" \
                  "The Pitch Detector continuously shows the note being played, as well as the intonation.\n" \
                  "Green means you're in tune, yellow means you're a little bit out of tune, and red means\n" \
                  "you're completely out of tune.\n" \
                  "The bottom area shows your recently played notes, represented on a piano octave.\n" \
                  "The Session Diagnostics area shows your overall score and actively reflected in the graph.\n" \
                  "You can click on \"More Info\" button to display how in tune you were for each note,\n" \
                  "the overall score, and how out of tune you were overall in cents.\n" \
                  "\n" \
                  "Keyboard Shortcuts:\n" \
                  "------------------------------------------\n" \
                  "Space: Pause or resume the session.\n" \


        tutorial_label = tk.Label(tutorial_window, text=message, font=("Calibri", 12), justify=tk.LEFT)
        tutorial_label.config(bg=background_color, fg="white")
        tutorial_label.pack()
        tutorial_window.lift(mainWindow.master)



# main gui for TuneCoach. Made by the group, Jamm Hostetler, James Eschrich, Joe Gravelle, Jenny Baik, Gavin Gui
from TuneCoach.gui.PitchDisplay import *
from TuneCoach.gui.Session import Session, load_session, save_session
from TuneCoach.gui.SessionHistory import *
from TuneCoach.gui.SessionDiagnostics import *
from TuneCoach.gui.RemoveWindow import *
from TuneCoach.gui.TunerSettingsWindow import *
from TuneCoach.gui.FAQWindow import *
from TuneCoach.gui.TutorialWindow import *
from TuneCoach.gui.IntroWindow import *


from tkinter import messagebox

def invalid_path(path): return path == '' or path == () or path == None

# Main GUI
class MainWindow:
    def __init__(self, master):
        self.practiceSessionList = []
        self.session = Session(SessionData(15), None)  # Temporary session! TODO: don't hardcode threshold
        self.audio_manager = AudioManager(self.session.data)
        self.threshold = 15
        self.yellow_threshold = 35

        self.paused = True
        self.master = master

        master.attributes('-fullscreen', True)
        master.state('iconic')
        self.screen_width = master.winfo_screenwidth()
        self.screen_height = master.winfo_screenheight()
        master.attributes('-fullscreen', False)
        master.deiconify()
        master.title("TuneCoach")
        master.geometry(f'{self.screen_width}x{self.screen_height}')
        master.minsize(width=int(self.screen_width/2), height=int(self.screen_height/2))
        master.maxsize(width=self.screen_width, height=self.screen_height)

        self.create_menubar()
        self.layout_frames(self.screen_width, self.screen_height)
        IntroWindow(self)
    
    # Creating frames to organize the screen.
    def layout_frames(self, screen_width, screen_height):
        self.bottom_frame = tk.Frame(self.master, bd=5, relief=tk.RAISED, bg=background_color)
        self.left_frame = tk.Frame(self.master, bd=5, relief=tk.RAISED, bg=background_color)
        self.right_frame = tk.Frame(self.master, bd=5, relief=tk.RAISED, bg=background_color)

        # Putting the frames into a grid layout
        self.bottom_frame.grid(row=1, column=0, columnspan=2, sticky="nsew")
        self.left_frame.grid(row=0, column=0, sticky="nsew")
        self.right_frame.grid(row=0, column=1, sticky="nsew")

        # setting up grid weights.
        self.master.grid_rowconfigure(0, weight=1)
        self.master.grid_rowconfigure(1, weight=1)
        self.master.grid_columnconfigure(0, weight=3)
        self.master.grid_columnconfigure(1, weight=4)

        # Here we can work on creating the functionality for each frame, ex: tuner, pitch history, information
        self.history = SessionHistory(self, self.bottom_frame)
        self.diagnostics = SessionDiagnostics(self)
        self.pitch_display = PitchDisplay(self)
    
    def save_as_practice_session(self):
        path = tk.filedialog.asksaveasfilename(initialdir = './', title="Save session as...", filetypes = [('session files', '*.session')])
        if invalid_path(path): # if the user cancels the dialog, don't do anything
            return False # we didn't save
        self.session = self.session.with_path(path)
        # notify session name change
        self.diagnostics.session_name.configure(text=self.session.name)
        save_session(self.session)
        return True # we did save
        
    # Adding menu options to the top of the screen.
    # returns False ONLY IF THE USER CANCELS
    def save_practice_session(self, ask=False):
        if self.session.path is None:

            # if ask if true, cancel if we say no at the messagebox or if we
            # cancel out of save_as. if ask is false, cancel only if we cancel out
            # of save_as
            if ask and not messagebox.askyesno("", "Save current session?"):
                return True # "save" was successfully, because the user chose to not save
            if not self.save_as_practice_session():
                return False # cancel save
        else:
            if ask and not messagebox.askyesno("", "Save current session?"):
                return True # "save" was successfully, because the user chose to not save
            save_session(self.session)
        
        messagebox.showinfo("Session Saved", f'Session saved to "{self.session.path}" successfully')
        return True # saved successfully

    def remove_practice_session(self):
        RemoveWindow(self)

    def tuner_settings(self):
        TunerSettingsWindow(self)

    def load_FAQ(self):
        FAQWindow(self)

    def load_tutorial(self):
        TutorialWindow(self)

    def setup_session(self, session):
        self.reset_everything()

        if self.audio_manager is not None:
            self.audio_manager.kill()
        
        self.session = session
        self.audio_manager = AudioManager(self.session.data)
        self.diagnostics.session_name.configure(text=self.session.name)

        if not self.session.data.empty:
            self.diagnostics.update_plot()
            self.history.update(self.session.data, force=True)

    def new_practice_session(self, ask=True):
        if ask:
            self.save_practice_session(ask=True)
        data = SessionData(self.threshold)
        self.setup_session(Session(data))
    
    def load_practice_session(self, ask=True):
        if ask:
            self.save_practice_session(ask=True)
        path = tk.filedialog.askopenfilename(initialdir = './', title="Select a session", filetypes = [('session files', '*.session')])
        if invalid_path(path): # if the user cancels the dialog, don't do anything
            return

        session = load_session(path)

        if session is None:
            messagebox.showerror("Invalid session", f'Session located at "{path}" is invalid!')
        else:
            self.setup_session(session)

    def create_menubar(self):
        menubar = tk.Menu(self.master)
        self.master.config(menu=menubar)
        file_menu = tk.Menu(menubar)

        # File menubar
        menubar.add_cascade(label="File", menu=file_menu)
        file_menu.add_command(label="New Practice Session", command=self.new_practice_session)
        file_menu.add_command(label="Save Current Session", command=self.save_practice_session)
        file_menu.add_command(label="Save Current Session As...", command=self.save_as_practice_session)
        file_menu.add_command(label="Load Existing Session", command=self.load_practice_session)
        # TODO: Add functionality to remove sessions
        # file_menu.add_separator
        # file_menu.add_command(label = "Remove Practice Session", command = self.remove_practice_session)

        # Settings menubar
        settings_menu = tk.Menu(menubar)
        menubar.add_cascade(label="Settings", menu=settings_menu)
        settings_menu.add_command(label="Tuner Settings", command=self.tuner_settings)

        # Help menubar
        help_menu = tk.Menu(menubar)
        menubar.add_cascade(label="Help", menu=help_menu)
        help_menu.add_command(label="FAQ", command=self.load_FAQ)
        help_menu.add_command(label="Tutorial", command=self.load_tutorial)

    def toggle_pause(self):
        if self.audio_manager is not None:
            if self.audio_manager.is_paused():
                print("Resuming")
                self.paused = False
                self.pitch_display.resume()
                self.audio_manager.resume()
                self.piano_update()
                self.score_update()
            else:
                print("Pausing")
                self.paused = True
                self.pitch_display.pause()
                self.audio_manager.pause()

    def force_pause(self):
        if self.audio_manager is not None and not self.audio_manager.is_paused():
            print("Pausing")
            self.paused = True
            self.pitch_display.pause()
            self.audio_manager.pause()

    def reset_everything(self):
        self.force_pause()
        self.history.clear()
        self.diagnostics.clear_plot()
    
    def score_update(self):
        if self.audio_manager is not None and not self.paused:
            self.diagnostics.update_plot()
        if not self.paused:
            self.master.after(500, lambda: self.score_update())

    def piano_update(self):
        self.history.update(self.session.data)
        if not self.paused:
            self.master.after(20, lambda: self.piano_update())



from TuneCoach.gui.MoreInfoWindow import *
import tkinter as tk
from TuneCoach.gui.constants import *
from TuneCoach.gui.ScoreLabel import *
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg


class SessionDiagnostics:
    @staticmethod
    def more_info_window_caller(mainWindow):
        MoreInfoWindow(mainWindow)

    def clear_plot(self):
        self.plot.clear()
        self.plot.set_xlim([0, 10])
        self.plot.set_ylim([0, 100])
        self.plot.set_autoscale_on(False)
        self.plot.set_title("Score Over Time")
        if self.mainWindow.session.data is not None:
            self.overallScoreLabel.set_text("Overall Score: %.2f" % self.mainWindow.session.data.get_overall())
        else:
            self.overallScoreLabel.set_text("N/A")
        self.canvas.draw()

    def update_plot(self):
        if self.mainWindow.session.data is not None:
            new_score = self.mainWindow.session.data.get_overall()
            self.overallScoreLabel.set_text("Overall Score: %.2f" % new_score)
            self.mainWindow.session.data.update_score_history()
            self.clear_plot()
            numScores = len(self.mainWindow.session.data._score_history)
            self.plot.plot(range(numScores), self.mainWindow.session.data._score_history, color="blue")
            self.canvas.draw()
        else:
            self.clear_plot()
            self.canvas.draw()

    def __init__(self, mainWindow):
        self.mainWindow = mainWindow
        workingFrame = mainWindow.left_frame
        currentSession = mainWindow.session.data
        topest_frame = tk.Frame(workingFrame, bd=5, bg=background_color)
        top_frame = tk.Frame(workingFrame, bd=5, bg=background_color)
        right_frame = tk.Frame(workingFrame, bd=5, bg=background_color)
        middle_frame = tk.Frame(workingFrame, bd=5, bg=background_color)
        bottom_frame = tk.Frame(workingFrame, bd=5, bg=background_color)

        topest_frame.grid(row=0, column=0, columnspan=2, sticky="nsew")
        top_frame.grid(row=1, column=0, sticky="nsew")
        right_frame.grid(row=1, column=1, rowspan=3, sticky="nsew")
        middle_frame.grid(row=2, column=0, sticky="nsew")
        bottom_frame.grid(row=3, column=0, sticky="nsew")

        workingFrame.grid_rowconfigure(0, weight=1)
        workingFrame.grid_rowconfigure(1, weight=1)
        workingFrame.grid_rowconfigure(2, weight=1)
        workingFrame.grid_rowconfigure(3, weight=1)
        workingFrame.grid_columnconfigure(0, weight=1)
        workingFrame.grid_columnconfigure(1, weight=2)

        title_label = tk.Label(topest_frame, text="Session Diagnostics", bg=background_color, fg="white",
                               font=("calibri", 20))
        title_label.pack(side=tk.TOP)
        self.session_name = tk.Label(topest_frame, text=mainWindow.session.name, bg=background_color,
                                    fg="light sky blue", font=("Calibri", 16))
        self.session_name.pack(side=tk.BOTTOM)

        if currentSession is None:
            print("There is no session.")
            v = "Overall Score: 0.00"
        else:
            v = "Overall Score: %.2f" % currentSession.get_overall()

        self.overallScoreLabel = ScoreLabel(top_frame, v, 150, 60)
        self.overallScoreLabel.pack()
        more_info_button = tk.Button(middle_frame, text="More info",
                                     command=lambda: self.more_info_window_caller(mainWindow))
        more_info_button.pack()

        defaultX = [0]
        defaultY = [0]
        if mainWindow.screen_width > 1000:
            self.fig = Figure(figsize=(3, 3))
        else:
            self.fig = Figure(figsize=(2,2))
        self.plot = self.fig.add_subplot(111)
        self.plot.plot(defaultX, defaultY, color='blue')

        # Not sure whether or not we want it to have the same axis the whole time
        self.plot.set_ylim([0, 100])
        self.plot.set_xlim([0, 10])
        my_fontsize = 16
        my_axissize = 14
        #if mainWindow.screen_width < 1000:
        #    my_fontsize = 3
        #    my_axissize = 3
        self.plot.set_title("Score Over Time", fontsize=my_fontsize)
        self.plot.set_ylabel("Score", fontsize=my_axissize)
        self.plot.set_autoscale_on(False)

        self.canvas = FigureCanvasTkAgg(self.fig, master=right_frame)
        self.canvas.get_tk_widget().configure(relief=tk.RIDGE, bd=5)
        self.canvas.get_tk_widget().pack()
        self.canvas.draw()



from ctypes import *
import threading
from TuneCoach.python_bridge.pitch_utilities import *
from TuneCoach.python_bridge.SessionData import *
from TuneCoach.pitch_detection import TunerStream
from time import sleep


class AudioThread(threading.Thread):
    def __init__(self, stream):
        super().__init__()
        self._stream = stream
        # self.daemon = True

    def run(self):
        print("Starting background audio thread")
        self._stream.mainloop()
        print("Exited background audio thread")


class Reader(threading.Thread):
    def __init__(self, stream, session):
        super().__init__()
        self._stream = stream
        # self.daemon = True
        if session is None:
            raise AttributeError('Session cannot be None!')
        self.session = session

    def run(self):
        print("Starting reader")
        while (self._stream.is_alive()):
            response, success = self._stream.read()
            if success and response:
                hz = response
                self.session.collect_data(hz)
            sleep(.01)
        print("Reader stopped")


class AudioManager(TunerStream):
    def __init__(self, session):
        super().__init__(44100)
        self._background_audio = threading.Thread(target=lambda: self.mainloop())
        self._background_audio.start()
        self._background_reader = Reader(self, session)
        self._background_reader.start()

    def destroy(self):
        self.kill()

    @property
    def paused(self):
        return self.is_paused()


import tkinter as tk
from TuneCoach.gui.constants import *


# Settings window to end current session
class EndSessionWindow:
    def __init__(self, mainWindow):
        if mainWindow.currentPracticeSession is not None:
            mainWindow.reset_everything()
            mainWindow.currentPracticeSession = None
            mainWindow.audio_manager.destroy()
            mainWindow.audio_manager = None
            mainWindow.myHistoryObject.clear()
            mainWindow.myDiagnosticObject.sessionName.configure(text="No Practice Session Selected")
        else:
            pass

        self.end_sesh_window = tk.Toplevel(mainWindow.master)
        self.end_sesh_window.configure(bg=background_color)
        self.end_sesh_window.geometry("200x100")

        top_frame = tk.Frame(self.end_sesh_window, bg=background_color, bd=5)
        bottom_frame = tk.Frame(self.end_sesh_window, bg=background_color, bd=5)
        top_frame.grid(row=0, sticky="nsew")
        bottom_frame.grid(row=1, sticky="nsew")

        self.end_sesh_window.grid_rowconfigure(0, weight=1)
        self.end_sesh_window.grid_rowconfigure(1, weight=1)

        success_label = tk.Label(top_frame, text="Session ended successfully.", fg="white", bg=background_color)
        success_label.pack()

        end_button = tk.Button(bottom_frame, text="Ok", command=lambda: self.end_sesh_window.destroy())
        end_button.pack()



import tkinter as tk
from TuneCoach.gui.constants import *


class FAQWindow:
    def __init__(self, mainWindow):
        faq_window = tk.Toplevel(mainWindow.master)
        faq_window.geometry()

        message = "Q: How do I use TuneCoach?\n" \
                  "A: Thorough instructions can be found in \"Tutorial\" under the \"Help\" dropdown menu.\n" \
                  "\n" \
                  "Q: Is there any way to change the intonation threshold?\n" \
                  "A: Yes! It is located in the \"Settings\" dropdown menu, then \"Tuner Settings\"\n" \
                  "\n" \
                  "Q: What instruments can be used with TuneCoach?\n" \
                  "A: Any instrument can be used with TuneCoach! Even your voice!\n" \
                  "\n" \
                  "Q: Why does the tuner not work?\n" \
                  "A: Check your microphone. Ensure that it is fully plugged in and is recognized as audio input.\n" \
                  "    Other apps and programs may be using the microphone, blocking access to TuneCoach.\n" \
                  "    Other apps must be closed in this case. If it does not help, restart TuneCoach or your PC.\n" \
                  "    Microphone permissions may also be blocked. To fix this, go into your system settings\n" \
                  "    to allow the microphone to be used with TuneCoach.\n" \
                  "\n" \
                  "Q: Do I need internet to use TuneCoach?\n" \
                  "A: No! Once TuneCoach is downloaded and installed, it can be used at any time!\n" \
                  "\n" \
                  "Q: Why am I not in tune?\n" \
                  "A: Git gud bro\n"

        faq_label = tk.Label(faq_window, text=message, font=("Calibri", 12), justify=tk.LEFT)
        faq_label.config(bg=background_color, fg="white")
        faq_label.pack()
        faq_window.lift(mainWindow.master)



import tkinter as tk
from TuneCoach.gui.constants import *
import tkinter.ttk as ttk


# Tuner settings window
class TunerSettingsWindow:
    def update_pitch_settings(self, cent_threshold, oldSettingsView):
        self.mainWindow.threshold = cent_threshold
        self.mainWindow.pitch_display.set_threshold(cent_threshold)
        self.mainWindow.session.data._threshold = cent_threshold

        oldSettingsView.destroy()

    def __init__(self, mainWindow):
        self.mainWindow = mainWindow
        tuner_settings_window = tk.Toplevel(self.mainWindow.master)
        tuner_settings_window.geometry("500x200")

        top_frame = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        middle_frame = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        middle_frame1 = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        middle_frame2 = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        # bottom_frame = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        # bottom_frame1 = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        # bottom_frame2 = tk.Frame(tuner_settings_window, bd=5, bg=background_color)
        bottomest_frame = tk.Frame(tuner_settings_window, bd=5, bg=background_color)

        # putting the frames into a grid layout
        top_frame.grid(row=0, column=0, columnspan=3, sticky="nsew")
        middle_frame.grid(row=1, column=0, columnspan=1, sticky="nsew")
        middle_frame1.grid(row=1, column=1, sticky="nsew")
        middle_frame2.grid(row=1, column=2, sticky="snew")
        # bottom_frame.grid(row=2, column=0, sticky="nsew")
        # bottom_frame1.grid(row=2, column=1, sticky="nsew")
        # bottom_frame2.grid(row=2, column=2, sticky="nsew")
        bottomest_frame.grid(row=2, column=0, columnspan=3, sticky="nsew")

        # setting up grid weights.

        tuner_settings_window.grid_rowconfigure(0, weight=1)
        tuner_settings_window.grid_rowconfigure(1, weight=5)
        tuner_settings_window.grid_rowconfigure(1, weight=5)
        # tuner_settings_window.grid_rowconfigure(3, weight=1)
        tuner_settings_window.grid_columnconfigure(0, weight=1)
        tuner_settings_window.grid_columnconfigure(1, weight=1)
        tuner_settings_window.grid_columnconfigure(2, weight=1)

        tuner_label = tk.Label(top_frame, text="Tuner Settings", font=("Calibri", 20))
        tuner_label.config(bg=background_color, fg="white")
        tuner_label.pack()

        centsitivity = tk.Label(middle_frame, text="Margin of Acceptable Pitch Error +- ")
        centsitivity.config(bg=background_color, fg="white")
        centsitivity.pack()

        v = tk.DoubleVar()
        v.set(mainWindow.threshold)

        cent_scale = tk.Scale(middle_frame1, from_=1, to=25, orient=tk.HORIZONTAL, variable=v)
        cent_scale.config(bg=background_color, fg="white")
        cent_scale.pack()

        in_cents = tk.Label(middle_frame2, text="cents")
        in_cents.config(bg=background_color, fg="white")
        in_cents.pack()

        done_button = ttk.Button(bottomest_frame, text="Apply",
                                command=lambda: self.update_pitch_settings(cent_scale.get(), tuner_settings_window))

        done_button.pack()
        tuner_settings_window.lift(self.mainWindow.master)



from tkinter import *
from tkinter.font import Font
import tkinter.ttk as ttk
from TuneCoach.python_bridge.pitch_utilities import *
from math import sin, cos, radians
import time
from TuneCoach.gui.indicatorlight import *


class PitchDisplay:
    def __init__(self, mainWindow, threshold=10):
        self.frame = mainWindow.right_frame
        self.mainWindow = mainWindow

        self.threshold = threshold

        self.font = Font(size=20)
        self.pitchOffset = self.font.metrics('linespace')/2

        self._pitchValue = '---'  # default display
        self._centsValue = -50
        self._hertzValue = 0
        self._octaveValue = ''

        self._span = 75  # Size of tuner arc in degrees, starting at vertical

        self.canvas = Canvas(self.frame)
        self.canvas.pack(fill=BOTH, expand=True)
        self.canvas.bind("<Configure>", self.configure)

        self.light = IndicatorLight(self.canvas, 35)
        self.light.pack(anchor='w', side='top')

        self.showsHertz = BooleanVar()

        c = ttk.Checkbutton(self.canvas, text="Show Hertz", variable=self.showsHertz, takefocus=False, command=self.display_default_gui)
        c.pack(anchor='e', side='bottom')

        self._last_time = 0
        self._clearing = False

        self.display_default_gui()
        self.update_data()

    def pause(self):
        self.light.stop()
        self.canvas.itemconfig(self.help_text, text='Press \'space\' to accept audio input')

    def resume(self):
        self.light.start_flashing()
        self.canvas.itemconfig(self.help_text, text='Press \'space\' to pause audio input')

    def cents_to_angle(self, cents):
        return cents/50 * self._span

    def configure(self, event):
        self.display_default_gui()

    def display_current_gui(self):
        pitch_and_octave = self._pitchValue + self._octaveValue
        self.canvas.itemconfig(self.current_pitch_display, text=pitch_and_octave)
        if self.showsHertz.get():
            self.canvas.itemconfig(self.hertzDisplay, text=self._hertzValue)
        self.update_line(self._centsValue)
        if not self._clearing and abs(self._centsValue) <= self.threshold:
            self.canvas.itemconfig(self.green_arc, fill="green")
        else:
            self.canvas.itemconfig(self.green_arc, fill="#ccffbf")

    def display_default_gui(self):
        self.canvas.delete("all")
        self.width = self.frame.winfo_width()
        self.height = self.frame.winfo_height()
        min_dimension = min(self.width, self.height)
        self.radius = 0.4 * min_dimension
        self.centerX = self.width/2
        self.centerY = self.height/2

        self.current_pitch_display = self.canvas.create_text(self.width/2, self.height/2 + self.pitchOffset, font=self.font, text='---')
        if self.showsHertz.get():
            self.hertzDisplay = self.canvas.create_text(self.width/2, self.height/2 + 3*self.pitchOffset, font=(None, 14), text='')

        self.help_text = self.canvas.create_text(self.width/2, self.height-25, text='Press \'space\' to accept audio input')

        x0 = self.centerX - self.radius
        y0 = self.centerY - self.radius
        x1 = self.centerX + self.radius
        y1 = self.centerY + self.radius

        # rect = self.canvas.create_rectangle(x0, y0, x1, y1)

        rStart = 90 - self._span
        rSpan = 2 * self._span
        yStart = 90 - self.cents_to_angle(self.mainWindow.yellow_threshold)
        ySpan = 2 * (90 - yStart)
        gStart = 90 - self.cents_to_angle(self.threshold)
        gSpan = 2 * self.cents_to_angle(self.threshold)

        self.red_arc = self.canvas.create_arc(x0, y0, x1, y1)
        self.canvas.itemconfig(self.red_arc, start=rStart, fill="#ffbfbf", extent=rSpan, outline='')

        self.yellow_arc = self.canvas.create_arc(x0, y0, x1, y1)
        self.canvas.itemconfig(self.yellow_arc,  start=yStart, fill="#fffeb0", extent=ySpan, outline='')

        self.green_arc = self.canvas.create_arc(x0, y0, x1, y1)
        self.canvas.itemconfig(self.green_arc, start=gStart, fill="#ccffbf", extent=gSpan, outline='')

        self.line = self.canvas.create_line(0, 0, 0, 0, fill="#452A23", width=4, 
                                            arrow=FIRST, arrowshape=(self.radius,10,5))
        self.update_line(-50)

    def update_line(self, cents):
        deg = self.cents_to_angle(cents)
        theta = radians(deg)
        dx = self.radius * sin(theta)
        dy = self.radius * cos(theta)
        self.canvas.coords(self.line, self.centerX, self.centerY, self.centerX + dx, self.centerY - dy)

    def set_threshold(self, thresh):
        self.threshold = thresh
        self.display_default_gui()

    def update_pitch(self, value):  # event as parameter
        self._pitchValue = value

    def update_hertz(self, value):
        self._hertzValue = value

    def update_cents(self, value):
        self._centsValue = value

    def update_octave(self, value):
        self._octaveValue = value

    def update_data(self):  # event
        if self.mainWindow.audio_manager is not None:
            hz = self.mainWindow.audio_manager.peek()
            if hz != 0:
                self._clearing = False
                midi = hz_to_midi(hz)
                pitch_class = midi_to_pitch_class(midi)
                desired_hz = closest_in_tune_frequency(hz)
                cent = cents(desired_hz, hz)
                name = pitch_class_to_name(pitch_class, Accidental.SHARP)  # TODO: coordinate accidental with Session
                self.update_cents(cent)
                self.update_hertz(f"{round(hz)} Hz")
                self.update_octave(f"{get_octave(midi)}")
                self.update_pitch(name)
                self.display_current_gui()
                self._last_time = time.time()
            else:
                self._clearing = True
                if self._centsValue != -50 and time.time() - self._last_time > 1.5:
                    self.update_cents(max(-50, self._centsValue - 3))
                    self.update_pitch('---')
                    self.update_hertz('')
                    self.update_octave('')
                    self.display_current_gui()

        self.mainWindow.master.after(10, self.update_data)



import collections
from TuneCoach.python_bridge.pitch_utilities import *
from TuneCoach.gui.MainWindow import *
import math
import datetime
import pickle
import os
from pathlib import Path

def save_to_file(session, path):
    with open(path, "wb") as file:
        pickle.dump(session, file)

def load_from_file(path):
    try:
        with open(path, "rb") as file:
            session = pickle.load(file)
            return session
    except Exception as e:
        print(e)
        return None

class SessionData:
    def __init__(self, cent_range):
        self._notes = ("C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B")
        self._in_tune_count = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        self._pitch_count = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        self._freq_history = []
        self._cents = 0.0
        self._overall = 0
        self._overall_count = 0
        self._threshold = cent_range
        self._timestamp = datetime.date.today()
        self._recent_notes = collections.deque([])
        self.display_buffer = collections.deque([])
        self.has_new_data = False

        # Potential data storage
        self._note_history = []
        self._cent_history = []
        self._score_history = []


    @staticmethod
    def get_existing_sessions():
        session_list = []
        for file in os.listdir("./user_sessions"):
            session_list.append(file)
        return session_list

    def get_overall(self):
        if self._overall_count == 0:
            return 0
        else:
            return (100.0 * self._overall) / self._overall_count

    def get_recent_notes(self):
        return self._recent_notes

    def update_threshold(self, new_threshold):
        self._threshold = new_threshold

    @property
    def empty(self):
        return self._overall_count == 0
    
    def update_score_history(self):
        new_score = self.get_overall()
        self._score_history.append(new_score)
        if len(self._score_history) > 10:
                self._score_history.pop(0)

    # Takes in frequency and calculates and stores all data
    def collect_data(self, hz):
        self.has_new_data = True
        self._freq_history.append(hz)
        midi = hz_to_midi(hz)
        index = midi_to_pitch_class(midi)
        desired_hz = closest_in_tune_frequency(hz)
        cent = cents(desired_hz, hz)
        name = pitch_class_to_name(index, Accidental.SHARP)
        octave = 2 + math.floor(math.log2(desired_hz / 65.4))

        # print(f"{name}{octave}: {round(hz, 2)} Hz ({round(cent)} cents)")

        # Gets counts of everything to calculate accuracy
        if abs(cent) <= self._threshold:
            self._in_tune_count[index] += 1
            self._overall += 1
        self._pitch_count[index] += 1
        self._overall_count += 1
        self._cents += abs(cent)

        self.display_buffer.append((name, cent))
        if len(self.display_buffer) > 64:
            self.display_buffer.popleft()

        # Only inserts a note if it's different than the last
        if len(self._recent_notes) == 0 or name != self._recent_notes[-1]:
            self._recent_notes.append(name)

        # If deque is full, pop
        if len(self._recent_notes) > 8:
            self._recent_notes.popleft()

    def display_all_data(self):
        print("These are your accuracies for each pitch class:")
        for i in range(12):
            if self._pitch_count[i] == 0:
                print(self._notes[i], "was not played/sung in the session.")
            else:
                pitch_error = (100.0 * self._in_tune_count[i]) / self._pitch_count[i]
                print("%s was in tune for %.2f %% of the time." % (self._notes[i], pitch_error))

        print("")
        if self._overall_count == 0:
            print("There was no audio input.")
        else:
            avg_cents = self._cents / self._overall_count
            print("Overall:")
            print("You were in tune for %.2f %% of the time." % self.get_overall())
            print("You were off by an an average of %.2f cents." % avg_cents)



import tkinter as tk
from TuneCoach.gui.constants import *


class SaveWindow: #TODO Gavin and Jenny work on storing data
    def __init__(self, mainWindow,x):
        #uses x to decide whether or not the session can actually be saved.
        myWindow = tk.Toplevel(mainWindow.master)
        myWindow.configure(bg = background_color)
        if x > 0:
            myLabel = tk.Label(myWindow, text = "Session Successfully Saved.", bg = background_color, fg = "white")
            myLabel.pack(side = tk.TOP, padx = 5, pady = 5)
        elif x < 0:
            myLabel = tk.Label(myWindow, text = "No Session to Save", fg = "white", bg = background_color)
            myLabel.pack(side = tk.TOP, padx = 5, pady = 5)
        else:
            myLabel = tk.Label(myWindow, text = "Can't Save Temporary Session", fg = "white", bg = background_color)
            myLabel.pack(side = tk.TOP, padx = 5, pady = 5)
        myButton = tk.Button(myWindow, text = "OK", command = lambda: myWindow.destroy())
        myButton.pack(side = tk.BOTTOM, padx = 5, pady = 5)
        myWindow.lift(mainWindow.master)



from TuneCoach.gui.NewSessionWindow import *
from TuneCoach.gui.LoadSessionWindow import *


class IntroWindow:
    @staticmethod
    def intro_load_session(mainWindow, oldFrame):
        oldFrame.destroy()
        mainWindow.load_practice_session(ask=False)

    @staticmethod
    def intro_new_session(mainWindow, oldFrame):
        oldFrame.destroy()
        mainWindow.new_practice_session(ask=False)

    def __init__(self, mainWindow):
        self.master = mainWindow.master
        introWindow = tk.Toplevel(self.master)
        introWindow.configure(bg=background_color)
        introWindow.geometry("400x250")

        topFrame = tk.Frame(introWindow, bg=background_color)
        middleFrame = tk.Frame(introWindow, bg=background_color)
        bottomFrame = tk.Frame(introWindow, bg=background_color)
        # bottomRight = tk.Frame(introWindow, bg=background_color)
        # bottomLeft = tk.Frame(introWindow, bg=background_color)

        topFrame.grid(row=0, column=1, sticky="nsew")
        middleFrame.grid(row=1, column=1, sticky="nsew")
        bottomFrame.grid(row=2, column=1, sticky="nsew")
        # bottomRight.grid(row=2, column=2, sticky="nsew")
        # bottomLeft.grid(row=2, column=0, sticky="nsew")

        introWindow.grid_rowconfigure(0, weight=1)
        introWindow.grid_rowconfigure(1, weight=1)
        introWindow.grid_rowconfigure(2, weight=1)
        # introWindow.grid_columnconfigure(0, weight=1)
        # introWindow.grid_columnconfigure(1, weight=1)
        # introWindow.grid_columnconfigure(2, weight=1)

        introLabel = tk.Label(topFrame, text="Welcome to Tune Coach!", anchor="e", fg="white", bg=background_color)
        introLabel.configure(font=("Calibri", 20))
        introLabel.pack(side=tk.TOP)

        text = "Please create a new session or load a previous session to begin using the application. \n" \
               "If you don't create or load a session, a temporary session will be used.\n" \
               "\n" \
               "For more information on using TuneCoach, navigate to the \"Help\" tab and then \"Tutorial\"."

        explainLabel = tk.Label(middleFrame, text=text, wraplength=350, bg=background_color, fg="white")
        explainLabel.pack()
        okButton = tk.Button(bottomFrame, text="Ok", command=lambda: introWindow.destroy())
        okButton.pack(side=tk.RIGHT, padx=20, pady=20)
        loadButton = tk.Button(bottomFrame, text="Load Session",
                               command=lambda: self.intro_load_session(mainWindow, introWindow))
        loadButton.pack(side=tk.LEFT, padx=20, pady=20)
        newSession = tk.Button(bottomFrame, text="New Session",
                               command=lambda: self.intro_new_session(mainWindow, introWindow))
        newSession.pack(side=tk.LEFT, padx=20, pady=20)

        introWindow.lift(self.master)



import tkinter as tk
from TuneCoach.gui.constants import *

class SaveProgressWindow:
    def save_session(self, mainWindow, oldWindow, callingFunction, callingWindow):
        oldWindow.destroy()
        mainWindow.save_practice_session()
        #then we know the function is loadSession, else its newSession
        if self.new_name is None:
            callingFunction.load_practice_session(callingWindow)
        else:
            callingFunction.creating_a_new_session(mainWindow, callingWindow, self.new_name)

    def no_save(self,mainWindow, oldWindow, callingFunction, callingWindow):
        oldWindow.destroy()
        if self.new_name is None:
            callingFunction.load_practice_session(callingWindow)
        else:
            callingFunction.creating_a_new_session(mainWindow, callingWindow, self.new_name)

    def __init__(self, mainWindow, callingFunction, callingWindow,newName = None):
        self.new_name = newName
        myWindow = tk.Toplevel(mainWindow.master)
        topFrame = tk.Frame(myWindow, bg = background_color)
        bottomFrame = tk.Frame(myWindow, bg = background_color)
        topFrame.grid(row = 0, sticky = "nsew")
        bottomFrame.grid(row = 1, sticky = "nsew")
        myWindow.grid_rowconfigure(0, weight = 1)
        myWindow.grid_rowconfigure(1, weight = 1)
        myLabel = tk.Label(topFrame, text = "Would you like to save your current session first?", bg = background_color, fg = "white")
        myLabel.pack()
        noButton = tk.Button(bottomFrame, text = "No", command = lambda: self.no_save(mainWindow, myWindow, callingFunction, callingWindow))
        noButton.pack(side = tk.LEFT, padx = 5, pady = 5)
        saveButton = tk.Button(bottomFrame, text = "Save", command = lambda: self.save_session(mainWindow, myWindow, callingFunction, callingWindow))
        saveButton.pack(side = tk.RIGHT, padx = 5, pady = 5)
        myWindow.lift(mainWindow.master)




import tkinter as tk

class ScoreLabel(tk.Canvas):
    def __init__(self, parent, text, width, height):
        super().__init__(parent, width=width, height=height, bg=parent["background"], bd=0, highlightthickness=0)
        self.backdrop = self.round_rectangle(0, 0, width, height, radius=40, fill='green')
        self._text = self.create_text(width/2, height/2, fill='white')
        self.itemconfig(self._text, text=text, font=(None, 10))
        self.set_text(text)

    def set_text(self, text):
        self.itemconfig(self._text, text=text)


    # thanks https://stackoverflow.com/questions/44099594/how-to-make-a-tkinter-canvas-rectangle-with-rounded-corners
    def round_rectangle(self, x1, y1, x2, y2, radius=25, **kwargs):
        points = [x1+radius, y1,
              x1+radius, y1,
              x2-radius, y1,
              x2-radius, y1,
              x2, y1,
              x2, y1+radius,
              x2, y1+radius,
              x2, y2-radius,
              x2, y2-radius,
              x2, y2,
              x2-radius, y2,
              x2-radius, y2,
              x1+radius, y2,
              x1+radius, y2,
              x1, y2,
              x1, y2-radius,
              x1, y2-radius,
              x1, y1+radius,
              x1, y1+radius,
              x1, y1]
        return self.create_polygon(points, **kwargs, smooth=True)


# code adapted from https://packaging.python.org/tutorials/packaging-projects/
from setuptools import setup, find_packages, Extension
from os.path import join

with open("README.md", "r") as fh:
    long_description = fh.read()

sources = ['bridge.cpp', 'TunerStream.cpp', 'PitchDetector.cpp', 'processing_utilities.cpp']
sources_root = './TuneCoach/pitch_detection'
pitch_detection = Extension('TuneCoach.pitch_detection', 
    sources = [join(sources_root, stem) for stem in sources],
    include_dirs = [sources_root],
    libraries = ['pulse-simple'],
    define_macros = [('USE_PULSE', '1')])

setup(
    name="TuneCoach",
    version="0.0.1",
    author="Jamm Hostetler , James Eschrich, Joe Gravelle, Jenny Baik, Gavin Gui",
    author_email="jeschrich@ufl.edu",
    description="An interactive tool for practicing intonation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bhostetler18/TuneCoach/",
    packages=['TuneCoach.gui', 'TuneCoach.python_bridge'],
    py_modules=["TuneCoach.main"],
    classifiers=[
        "Programming Language :: Python :: 3",
        # "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    ext_modules=[pitch_detection],
    python_requires='>=3.8',
    install_requires = [
        'numpy>=1.19.0',
        'matplotlib>=3.3.0rc1',
        'Pillow>=7.1.2'
    ],
    include_package_data=True,
    entry_points = {
        'console_scripts': ['TuneCoach=TuneCoach.main:main'],
    }
)



import tkinter as tk


class IndicatorLight(tk.Canvas):
	def __init__(self, parent, size=20):
		super().__init__(parent, width=size, height=size)
		self._parent = parent
		self.light = self.create_oval(0.15*size, 0.15*size, 0.85*size, 0.85*size, fill='red', width=0, state=tk.HIDDEN)
		self._isOn = False
		self._flashing = False
		self._job = None

		self.pausedCircle = self.create_oval(0, 0, size, size, fill='#72d2e0', width=0)
		self.line1 = self.create_line(0.35*size, 0.35*size, 0.35*size, 0.65*size, width=size/4.5, capstyle=tk.ROUND)
		self.line2 = self.create_line(0.65*size, 0.65*size, 0.65*size, 0.35*size, width=size/4.5, capstyle=tk.ROUND)

	def start_flashing(self):
		if not self._flashing:
			self._flashing = True
			self.itemconfig(self.pausedCircle, state=tk.HIDDEN)
			self.itemconfig(self.line1, state=tk.HIDDEN)
			self.itemconfig(self.line2, state=tk.HIDDEN)
			self._flash()

	def _flash(self):
		if self._flashing:
			state = tk.HIDDEN if self._isOn else tk.NORMAL
			self.itemconfig(self.light, state=state)
			self._isOn = not self._isOn
			self._job = self._parent.after(800, self._flash)

	def stop(self):
		if self._job is not None:
			self._parent.after_cancel(self._job)
		self._flashing = False
		self.itemconfig(self.light, state=tk.HIDDEN)
		self.itemconfig(self.pausedCircle, state=tk.NORMAL)
		self.itemconfig(self.line1, state=tk.NORMAL)
		self.itemconfig(self.line2, state=tk.NORMAL)
		self._isOn = False



import math
from enum import Enum


class Accidental(Enum):
    FLAT = 0
    SHARP = 1


def hz_to_midi(hz):
    return int(round(12.0 * math.log(hz/440.0, 2) + 69.0))


def midi_to_hz(midi):
    return 440.0 * 2.0**((midi - 69)/12.0)


def midi_to_pitch_class(midi):
    return midi % 12


def get_octave(midi):
    return int(midi/12) - 1


def pitch_class_to_name(pitch, acc):
    flat = ["C", "Db", "D", "Eb", "E", "F", "Gb", "G", "Ab", "A", "Bb", "B"]
    sharp = ["C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"]
    if acc == Accidental.FLAT:
        return flat[pitch]
    elif acc == Accidental.SHARP:
        return sharp[pitch]


def cents(target, actual):
    return 1200.0 * math.log(actual/target, 2)


def closest_in_tune_frequency(hz):
    return midi_to_hz(hz_to_midi(hz))


import tkinter as tk
from TuneCoach.gui.constants import *
from TuneCoach.gui.NewSessionWindow import *
from TuneCoach.python_bridge.SessionData import SessionData
from TuneCoach.gui.SaveProgressWindow import *


# Settings window to load new session
class LoadSessionWindow:
    #will see if you want to save your current session before loading new session, then call load_practice_session
    def propose_save(self, oldWindow):
        if self.mainWindow.currentPracticeSession is not None and self.mainWindow.currentPracticeSession._name != "Temporary Session":
            SaveProgressWindow(self.mainWindow, self, oldWindow)
        else:
            self.load_practice_session(oldWindow)
    #actually loads the practice session
    def load_practice_session(self, oldWindow):
        path = tk.filedialog.askopenfilename(initialdir = './', title="Select a session", filetypes = (('Session files', '*.session')))
        if path is None: # if the user cancels the dialog, don't do anything
            return
        session = SessionData.load_from_file(path, self.selected_session.get())
        if session is None:
            pass  # TODO: Handle error, display to user
        else:
            self.mainWindow.currentPracticeSession = session
            if self.mainWindow.audio_manager is not None:
                self.mainWindow.audio_manager.destroy()
            self.mainWindow.reset_everything()
            self.mainWindow.audio_manager = AudioManager(session)
            self.mainWindow.myDiagnosticObject.sessionName.configure(text=session._name)
            oldWindow.destroy()

    def load_new_session(self, oldWindow):
        oldWindow.destroy()
        NewSessionWindow(self.mainWindow)

    def __init__(self, mainWindow):
        self.mainWindow = mainWindow
        self.master = mainWindow.master

        load_window = tk.Toplevel(self.master)
        top_frame = tk.Frame(load_window, bd=5, bg=background_color)
        left_frame = tk.Frame(load_window, bd=5, bg=background_color)
        middle_frame = tk.Frame(load_window, bd=5, bg=background_color)
        right_frame = tk.Frame(load_window, bd=5, bg=background_color)

        top_frame.grid(column=0, row=0, columnspan=3, sticky="nsew")
        left_frame.grid(row=1, column=0, sticky="nsew")
        middle_frame.grid(row=1, column=1, sticky="nsew")
        right_frame.grid(row=1, column=2, sticky="nsew")

        load_window.grid_rowconfigure(0, weight=1)
        load_window.grid_rowconfigure(1, weight=3)
        load_window.grid_columnconfigure(0, weight=1)
        load_window.grid_columnconfigure(1, weight=1)
        load_window.grid_columnconfigure(2, weight=1)

        create_session_label = tk.Label(top_frame, text="Load Previous Session", bg=background_color, fg="white")
        create_session_label.pack()

        session_files = SessionData.get_existing_sessions()
        session_names = list(map(lambda filename: os.path.splitext(filename)[0], session_files))
        if len(session_files) > 0:
            select_session_label = tk.Label(left_frame, text="Select a session to load", bg=background_color, fg="white")
            select_session_label.pack()
            self.selected_session = tk.StringVar()
            self.selected_session.set(session_files[0]) # strip out extension

            load_session_dropdown = tk.OptionMenu(middle_frame, self.selected_session, *session_files)
            load_session_dropdown.pack()
            #TODO: complete functionality for acceptButton
            acceptButton = tk.Button(right_frame, text = "Select", command = lambda: self.propose_save(load_window))
            acceptButton.pack()
        else:
            stand_in_label = tk.Label(middle_frame, text="No sessions to choose from. \n Create a new session first.", fg="white", bg=background_color)
            stand_in_label.pack()
            new_session_button = tk.Button(right_frame, text = "New Session", command = lambda: self.load_new_session(load_window))
            new_session_button.pack() 

        load_window.lift(self.master)
